/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "WSIF" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, 2002, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package com.myeis;

import java.io.*;
import org.apache.wsif.providers.jca.toolplugin.*;
import com.myeis.repository.*;
import com.myeis.services.*;
import com.myeis.services.internal.*;

public class MyEIS {

	public byte[] doIt(byte[] input) {
		
		/*
		 * This method represents the actual sample backend (i.e. this method mimics what a real back end system would typically do).
		 * Based on the function name certain tasks are executed.
		 */
	
		try {
			ByteArrayInputStream inputStream = new ByteArrayInputStream(input);
			
			// get the function name
			ObjectInputStream headerInputStream = new ObjectInputStream(inputStream);
			String functionName = (String)headerInputStream.readObject();
		
			if (functionName.equals("IMPORT_PORTTYPES")) {
				// import service porttypes
				ObjectInputStream payloadInputStream = new ObjectInputStream(inputStream);
				String queryString = (String)payloadInputStream.readObject();
				PortTypeArray portTypeArray = (new Repository()).getPortTypes(queryString);
				
				ByteArrayOutputStream outputStream = new ByteArrayOutputStream();		
		        ObjectOutputStream objectOutputStream = new ObjectOutputStream(outputStream);
  				objectOutputStream.writeObject(portTypeArray);
		        objectOutputStream.flush();					
			
				return outputStream.toByteArray();
			}
			if (functionName.equals("IMPORT_DEFINITION")) {
				// import service definitions
				ObjectInputStream payloadInputStream = new ObjectInputStream(inputStream);				
				PortTypeSelection selection = (PortTypeSelection)payloadInputStream.readObject();
				ImportDefinition importDefinition = (new Repository()).getDefinition(selection);
			
				ByteArrayOutputStream outputStream = new ByteArrayOutputStream();		
		        ObjectOutputStream objectOutputStream = new ObjectOutputStream(outputStream);
  				objectOutputStream.writeObject(importDefinition);
		        objectOutputStream.flush();					
			
				return outputStream.toByteArray();
			}
			if (functionName.equals("IMPORT_RAWMETADATA")) {
				// import raw metadata
				ObjectInputStream payloadInputStream = new ObjectInputStream(inputStream);
				String queryString = (String)payloadInputStream.readObject();
				byte[] result  = (new Repository()).getRawEISMetaData(queryString);
				
				return result;
			}
			if (functionName.equals("CUSTOMERINFO_getCustomer")) {
				// invoke a business service
				ObjectInputStream payloadInputStream = new ObjectInputStream(inputStream);				
				String number = (String)payloadInputStream.readObject();
				CustomerDataObject customer = (new CustomerInfo()).getCustomer(number);
			
				ByteArrayOutputStream outputStream = new ByteArrayOutputStream();		
		        ObjectOutputStream objectOutputStream = new ObjectOutputStream(outputStream);
  				objectOutputStream.writeObject(customer);
		        objectOutputStream.flush();					
			
				return outputStream.toByteArray();
			}			
			if (functionName.equals("PURCHASEORDERINFO_getPurchaseOrder")) {
				// invoke a business service
				ObjectInputStream payloadInputStream = new ObjectInputStream(inputStream);				
				String number = (String)payloadInputStream.readObject();
				PurchaseOrderDataObject purchaseOrder = (new PurchaseOrderInfo()).getPurchaseOrder(number);
			
				ByteArrayOutputStream outputStream = new ByteArrayOutputStream();		
		        ObjectOutputStream objectOutputStream = new ObjectOutputStream(outputStream);
  				objectOutputStream.writeObject(purchaseOrder);
		        objectOutputStream.flush();					
			
				return outputStream.toByteArray();
			}			
			
			if (functionName.equals("CUSTOMERINFO_getAddress")) {
				// invoke a business service
				ObjectInputStream payloadInputStream = new ObjectInputStream(inputStream);				
				String number = (String)payloadInputStream.readObject();
				AddressDataObject address = (new CustomerInfo()).getAddress(number);
			
				ByteArrayOutputStream outputStream = new ByteArrayOutputStream();		
		        ObjectOutputStream objectOutputStream = new ObjectOutputStream(outputStream);
  				objectOutputStream.writeObject(address);
		        objectOutputStream.flush();					
			
				return outputStream.toByteArray();
			}			
			// process other business services here
		
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return null;
	}

}

