/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "WSIF" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, 2002, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package providers;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.wsif.providers.soap.apacheaxis.WSIFDynamicProvider_ApacheAxis;
import org.apache.wsif.providers.soap.apachesoap.WSIFDynamicProvider_ApacheSOAP;
import org.apache.wsif.spi.WSIFProvider;
import org.apache.wsif.util.WSIFPluggableProviders;
import util.TestUtilities;

/**
 * Junit test to test the plugable providers facility for WSIFDynamicProviders
 */
public class PlugableProvidersTest extends TestCase {

    private static final Class DEFAULT_SOAP_PROVIDER = 
       TestUtilities.DEFAULT_SOAP_PROVIDER_CLASS;       
    private static final Class NON_DEFAULT_SOAP_PROVIDER = 
       TestUtilities.NON_DEFAULT_SOAP_PROVIDER_CLASS;       

    private static final String DEFAULT_PROVIDER_NAME =
       DEFAULT_SOAP_PROVIDER.getName(); 
    private static final String NON_DEFAULT_PROVIDER_NAME =
       NON_DEFAULT_SOAP_PROVIDER.getName(); 

    public PlugableProvidersTest(String name) {
    	 super(name);
    }
        
	public static void main(String[] args) {
		junit.textui.TestRunner.run (suite());
	}
		
	public static Test suite() {
		return new TestSuite(PlugableProvidersTest.class);
	}
	  
    public void setUp() {
       TestUtilities.setUpExtensionsAndProviders();
    }

    public void testEJBProvider() { 
    	reset();
    	assertTrue( "testEJBProvider doit failed!", doit("http://schemas.xmlsoap.org/wsdl/ejb/",
    	     "org.apache.wsif.providers.ejb.WSIFDynamicProvider_EJB") );
    }
        	     
    public void testJavaProvider() { 
    	reset();
    	assertTrue( "testJavaProvider doit failed!", doit("http://schemas.xmlsoap.org/wsdl/java/",
    	     "org.apache.wsif.providers.java.WSIFDynamicProvider_Java") );
    }
    	     
    public void testDefaultSoapProvider() { 
    	reset();

     	assertTrue( 
     	   "testDefaultSoapProvider doit failed!", 
     	   doit(
     	      "http://schemas.xmlsoap.org/wsdl/soap/",
    	      DEFAULT_PROVIDER_NAME ) );
    }
    	     
    public void testSetDefaultSoapProvider() { 
       reset();
       WSIFProvider p = null;
       try {
          p = (WSIFProvider)NON_DEFAULT_SOAP_PROVIDER.newInstance();
       } catch (Exception ex) {
          assertTrue( "exception instantiating non default provider: " + ex.getMessage(), false );
       }
       WSIFPluggableProviders.overrideDefaultProvider( 
          "http://schemas.xmlsoap.org/wsdl/soap/",
          p );
       assertTrue( 
          "testSetDefaultSoapProvider", 
          doit(
             "http://schemas.xmlsoap.org/wsdl/soap/",
    	     NON_DEFAULT_PROVIDER_NAME ) );
       // set null so uses default next time
       WSIFPluggableProviders.overrideDefaultProvider( 
          "http://schemas.xmlsoap.org/wsdl/soap/", null );

    }

    //public void testSoapRMIProvider() { 
    //	assertTrue( doit("http://schemas.xmlsoap.org/wsdl/soap/xxx",
    //	     "org.apache.wsif.providers.soap.soaprmi.WSIFDynamicProvider_SoapRMI") );
    //}
    	     
    public void testPPMethods() { 

    	reset();
    
       // test isProviderAvailable method
       assertTrue( "isProviderAvailable 1", 
          WSIFPluggableProviders.isProviderAvailable( 
             "http://schemas.xmlsoap.org/wsdl/soap/",
             "http://schemas.xmlsoap.org/wsdl/soap/") );
       assertTrue( "isProviderAvailable 2", 
          WSIFPluggableProviders.isProviderAvailable( 
             "http://schemas.xmlsoap.org/wsdl/soap/",
             "http://schemas.xmlsoap.org/wsdl/jms/") );
       assertTrue( "isProviderAvailable 3", 
          !WSIFPluggableProviders.isProviderAvailable( 
             "http://schemas.xmlsoap.org/wsdl/soap/",
             "http://schemas.xmlsoap.org/wsdl/xxx/") );
       assertTrue( "isProviderAvailable 4", 
          !WSIFPluggableProviders.isProviderAvailable( 
             "http://schemas.xmlsoap.org/wsdl/xxx/",
             "http://schemas.xmlsoap.org/wsdl/jms/") );
       assertTrue( "isProviderAvailable 5", 
          !WSIFPluggableProviders.isProviderAvailable( 
             "http://schemas.xmlsoap.org/wsdl/xxx/",
             "http://schemas.xmlsoap.org/wsdl/xxx/") );
       assertTrue( "isProviderAvailable 6", 
          WSIFPluggableProviders.isProviderAvailable( 
             "http://schemas.xmlsoap.org/wsdl/java/") );
       assertTrue( "isProviderAvailable 7", 
          !WSIFPluggableProviders.isProviderAvailable( 
             "http://schemas.xmlsoap.org/wsdl/xxx/") );

       // test the default SOAP provider
       WSIFProvider p1 = WSIFPluggableProviders.getProvider(
          "http://schemas.xmlsoap.org/wsdl/soap/" );
       Class c1 = p1.getClass();
       assertTrue( "defaultprovider 0", DEFAULT_SOAP_PROVIDER.equals( c1 ) );

       // test changing the default provider
       WSIFProvider p = null;
       try {
          p = (WSIFProvider)NON_DEFAULT_SOAP_PROVIDER.newInstance();
       } catch (Exception ex) {
          assertTrue( "exception instantiating non default provider: " + ex.getMessage(), false );
       }
       WSIFPluggableProviders.overrideDefaultProvider( 
          "http://schemas.xmlsoap.org/wsdl/soap/",
          p );
          
       WSIFProvider p2 = WSIFPluggableProviders.getProvider(
          "http://schemas.xmlsoap.org/wsdl/soap/" );
       Class c2 = p2.getClass();
       assertTrue( "defaultprovider 1", NON_DEFAULT_SOAP_PROVIDER.equals( c2 ) );

       WSIFPluggableProviders.overrideDefaultProvider( 
          "http://schemas.xmlsoap.org/wsdl/soap/",
          null );
       WSIFProvider p3 = WSIFPluggableProviders.getProvider(
          "http://schemas.xmlsoap.org/wsdl/soap/" );
       Class c3 = p3.getClass();
       assertTrue( "defaultprovider 2", c1.equals( c3 ) );
       assertTrue( "defaultprovider 3", p1 == p3 );

       WSIFPluggableProviders.overrideDefaultProvider( 
          "http://schemas.xmlsoap.org/wsdl/soap/",
          p1 );
       p2 = WSIFPluggableProviders.getProvider(
          "http://schemas.xmlsoap.org/wsdl/soap/" );
       c2 = p2.getClass();
       assertTrue( "defaultprovider 4", c1.equals( c2 ) );
       assertTrue( "defaultprovider 5", p1 == p2 );

       WSIFPluggableProviders.overrideDefaultProvider( 
          "http://schemas.xmlsoap.org/wsdl/soap/",
          null );
       p2 = WSIFPluggableProviders.getProvider(
          "http://schemas.xmlsoap.org/wsdl/soap/" );
       c2 = p2.getClass();
       assertTrue( "defaultprovider 6", c1.equals( c2 ) );
       assertTrue( "defaultprovider 7", p1 == p2 );
                 
       // test provider auto loading
       WSIFPluggableProviders.setAutoLoadProviders( false );
       assertTrue( "AutoLoadProviders 1", 
          !WSIFPluggableProviders.isAutoLoadProviders() );
       assertTrue( "AutoLoadProviders 2", 
          !WSIFPluggableProviders.isProviderAvailable( 
             "http://schemas.xmlsoap.org/wsdl/soap/",
             "http://schemas.xmlsoap.org/wsdl/soap/") );
       WSIFPluggableProviders.overrideDefaultProvider( 
          "http://schemas.xmlsoap.org/wsdl/soap/",
          new WSIFDynamicProvider_ApacheAxis() );
       assertTrue( "AutoLoadProviders 3", 
          WSIFPluggableProviders.isProviderAvailable( 
             "http://schemas.xmlsoap.org/wsdl/soap/",
             "http://schemas.xmlsoap.org/wsdl/soap/") );
       WSIFPluggableProviders.setAutoLoadProviders( true );
       assertTrue( "AutoLoadProviders 4", 
          WSIFPluggableProviders.isAutoLoadProviders() );
          
       assertTrue( "AutoLoadProviders 5", doit(
          "http://schemas.xmlsoap.org/wsdl/soap/",
          DEFAULT_PROVIDER_NAME ) );

    }
        	     
    private boolean doit (String namespaceURI, String providerClassName) {  
       WSIFProvider provider = WSIFPluggableProviders.getProvider( namespaceURI );
       String pn = provider.getClass().getName();
       return pn.equals( providerClassName );
    }
    
    private void reset() {
       // reset and clear loaded providers
       WSIFPluggableProviders.setAutoLoadProviders( false );
       WSIFPluggableProviders.setAutoLoadProviders( true );
    }

}





