/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "WSIF" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, 2002, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package util;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.wsif.WSIFOperation;
import org.apache.wsif.base.WSIFDefaultPort;

/**
 * Junit test for the finalize method on the WSIFDefaultPort.
 * 
 * @author Ant Elder <ant.elder@uk.ibm.com>
 */
public class PortFinalizeTest extends TestCase {

  public PortFinalizeTest(String name) {
    super(name);
  }

  public static void main(String[] args) {
     junit.textui.TestRunner.run(suite());
  }

  public static Test suite() {
    return new TestSuite(PortFinalizeTest.class);
  }

  public void setUp() {
    TestUtilities.setUpExtensionsAndProviders();
  }

  public void testFinalize() throws Throwable {
  	DummyPort p = new DummyPort();
  	assertTrue( "freshly created port should not be closed!", !DummyPort.isClosed() );
  	//p = null;
    //System.gc();
    //System.runFinalization();
    // unfortunately the above code doesn't seem to work in WSAD5
    // which makes this test not so useful. TODO - How can I fix that?
  	p.finalize();
  	assertTrue( "port should now be closed!", DummyPort.isClosed() );
  }

}

class DummyPort extends WSIFDefaultPort {
	
	static private boolean closed;
	
    public DummyPort() {
    	closed = false;
    }
    
	public WSIFOperation createOperation(String s1) {
		return null;
	}

	public WSIFOperation createOperation(String s1, String s2, String s3) {
		return null;
	}

    public void close() {
    	closed = true;
    }
    
    static public boolean isClosed() {
    	return closed;
    }

}
